/*
 * Program: PasswordSyncLedger.java
 * Purpose: A ledger that listens to specific topic where the Password Synchronizer
 *          has published service requests to.
 * @author: Ray Lai (ray.lai@sun.com)
 * Updated: May 29, 2005, 2:22 PM
 * Copyright:  (c) 2006 by Ray Lai
 * Remarks:    Full program and documentation will be available under developer.java.net
 *             For simplicity, logger class is removed.
 * ============================================================================================
 */

package com.csp.provisioning;

import java.io.IOException;
import java.io.InputStreamReader;
import java.util.LinkedHashMap;
import javax.jms.JMSException;
import javax.jms.MessageListener;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.jms.Message;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicSubscriber;
import javax.xml.soap.MessageFactory;
import com.sun.messaging.TopicConnectionFactory;
import java.util.Date;

public class PasswordSyncLedger implements MessageListener {
    
    protected TopicConnectionFactory  topicConnectionFactory;
    protected TopicConnection         topicConnection;
    protected TopicSession            topicSession;
    protected Topic                   topic;
    protected TopicSubscriber         topicSubscriber;
    protected TextMessage             inMessage;
    protected TextMessage             message;
    protected Message                 receivedMessage;
    protected InputStreamReader       inputStreamReader;
    protected String                  topicName;
    protected char                    answer = '\0';
    protected final boolean           NOLOCAL = true;
    protected ServiceConfig serviceConfig;
    protected ServiceConfigContext context;
    protected LinkedHashMap<String,ServiceConfigContext> serviceConfigHashMap = new LinkedHashMap();
    protected MessageFactory messageFactory = null;
    protected String timeStamp;
    
    
    /**
     * Creates a new instance of PasswordSyncLedger 
     */
    public PasswordSyncLedger()  {
        // set default topic name
        this.topicName = "PWDSYNC_PROVIDER_OK_LIST";
        
        // load Password Synchronizer config file
        serviceConfig = new ServiceConfig();
        serviceConfigHashMap = serviceConfig.getAllConfigContext();
        
        System.out.println("Password Synchronizer Ledger starts.");
        
        // set up JMS connection factory
        init(this.topicName);
        start();
    }
    
    
   /* Set up JMS topic connection, and
    *    initialize the JMS set-up
    *
    * @exception ex Exception
    */
    public void init(String topicName) {
        
        try {
            this.topicConnectionFactory = new com.sun.messaging.TopicConnectionFactory();
            this.topicConnection = this.topicConnectionFactory.createTopicConnection();
            this.topicSession = this.topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
            this.topic = this.topicSession.createTopic(topicName);
            
            messageFactory = MessageFactory.newInstance();
        } // try
        catch (Exception ex) {
            ex.printStackTrace();
            System.out.println("Cannot create topics or topic names");
            System.out.println("Connection problem: " + ex.toString());
            if (topicConnection != null) {
                try {
                    topicConnection.close();
                } catch (JMSException moreEx) {
                    //
                } // catch
            } // if topicConnection
            System.exit(1);
        } // catch
    }
    
   /*
    * Process each message received
    *    from the listener.
    *
    * @exception ex JMSException
    */
    public void onMessage(Message message) {
        
        int foundAny = -1;
        
        try {
            TextMessage textMessage = (TextMessage) message;
            String syncResult = textMessage.getText();
            
            // assume the application is synchronized
            this.timeStamp = new Date().toString();
            System.out.println(this.timeStamp + "- just complete password synchronization for '" + syncResult + "'");
     
            //System.out.println(timeStamp + "- textMessage contents=" + syncResult);
            if (this.serviceConfig.getContext(syncResult).getApplicationId().equals(syncResult)) {
               //System.out.println("DEBUG - syncResult == ApplicationId");
                
                // set state to SYNC_STATE
               this.serviceConfig.getContext(syncResult).setState(ServiceConfigContext.SYNC_STATE);
               
               viewResult();
            }
            
        } // try
        catch (JMSException jmsex) {
            jmsex.printStackTrace();
        }  // catch
    }
    
   /*
    * Start listening to the topic (passed in args[0])
    *    under a loop. It will only stop when users press Q or q.
    *
    * @exception ex JMSException
    */
    public void start() {
        
        try {
            topicSubscriber = this.topicSession.createSubscriber(topic, null, NOLOCAL);
            this.topicSession.createSubscriber(this.topic);
            
            this.topicSubscriber.setMessageListener(this);
            this.topicConnection.start();
            
            System.out.println("Command Option : Q=quit, then <return>");
            inputStreamReader = new InputStreamReader(System.in);
            if ((answer == 'v') || (answer == 'V')) {
                viewResult();
            }
            while (!((answer == 'q') || (answer == 'Q'))) {
                try {
                    answer = (char) inputStreamReader.read();
                } catch (IOException e) {
                    e.printStackTrace();
                    System.out.println("I/O exception: " + e.toString());
                } // catch
            } // while
        } // try
        catch (JMSException ex) {
            ex.printStackTrace();
            System.out.println("Cannot subscribe message");
            System.out.println("Exception occurred: " + ex.toString());
        } finally {
            if (topicConnection != null) {
                try {
                    topicConnection.close();
                } catch (JMSException ex) {
                    ex.printStackTrace();
                    System.out.println("Cannot close topicConnection");
                    System.out.println("Connection problem: " + ex.toString());
                } // catch
            }  // if topicConnection
        }// finally
    }
    
    /**
     * verify whether all provisioning target systems are synchronized
     */
    private void viewResult() {
        int totalSync = 0;
        
        for(ServiceConfigContext configContext: this.serviceConfigHashMap.values()) {      
           if (configContext.getState() != (configContext.SYNC_STATE)) {
              totalSync++;
           }     
        }
        
        this.timeStamp = new Date().toString();
        if (totalSync == 0) {
            System.out.println(this.timeStamp + "- Notification - all passwords are synchronized in all systems.");
            System.out.println("Password Synchronizer Ledger is stopped.");
            System.exit(0);
        }
        else {
            System.out.println(this.timeStamp + "- " + totalSync + " applications need to be synchronized.");
        }
    }

    
    public static void main(String args[]) {
        new PasswordSyncLedger();
    }
}