/*
 * Program: PwdSyncListener.java
 * Purpose: JMS Listener for password synchronizer service requests
 * @author: Ray Lai (ray.lai@sun.com)
 * Updated: May 31, 2005, 9:29 AM
 * Copyright:  (c) 2006 by Ray Lai
 * Remarks:    Full program and documentation will be available under developer.java.net
 *             For simplicity, logger class is removed.
 * ============================================================================================
 * Summary:
 *
 * The Password Synchronizer listener listens to a specific topic PROD_FINANCIAL_FRONTOFFICE
 * If it receives any message request, it processes it (in this example, there will be no action.
 * After that, it publishes a job completion message to the topic PWDSYNC_PROVIDER_OK_LIST
 *
 * The topic PWDSYNC_PROVIDER_OK_LIST will be subscribed by PwdSyncLedger which will check
 * for completion result.
 *
 * Syntax:  java PasswordSyncListener <topic name> <notify topic name>
 * where
 *      topic name = topic name where PwdSyncListener will process password sync requests
 *      notify topic name = upon completion, PwdSyncListener will notify the request is done under
 *                          that topic
 *
 * if no param is given, the default will be:
 *      topic name = PROD_FINANCIAL_FRONTOFFICE
 *      notify topic name = PWDSYNC_PROVIDER_OK_LIST
 * ============================================================================================
 * Constraints - known bug in displaying the XML doc contents - some tags are not in order.
 */

package com.csp.provisioning;


import java.io.IOException;
import java.io.InputStreamReader;
import javax.jms.JMSException;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.jms.Message;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicSubscriber;
import javax.xml.soap.MessageFactory;
import com.sun.messaging.TopicConnectionFactory;
import com.sun.messaging.xml.MessageTransformer;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.Name;
import javax.xml.soap.Node;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPFactory;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPHeaderElement;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.Text;



public class PasswordSyncListener implements javax.jms.MessageListener {
    
    protected TopicConnectionFactory  topicConnectionFactory = null;
    protected TopicConnection         topicConnection = null;
    protected TopicSession            topicSession = null;
    protected Topic                   topic = null;
    protected TopicSubscriber         topicSubscriber = null;
    protected TextMessage             message = null;
    protected InputStreamReader       inputStreamReader = null;
    protected String                  topicName = "prod_application1";
    protected String                  notifyTopicName = "PWDSYNC_PROVIDER_OK_LIST";
    protected MessageFactory          messageFactory = null;
    protected String timeStamp;
    protected ServiceConfig serviceConfig;
    protected ServiceConfigContext context;
    protected LinkedHashMap<String,ServiceConfigContext> serviceConfigHashMap = new LinkedHashMap();
    
    /**
     * Constructor - create new instance of Password Synchronizer listener
     * This is a default constructor if no param is given at run-time
     */
    public PasswordSyncListener() {
        serviceConfig = new ServiceConfig();
        serviceConfigHashMap = serviceConfig.getAllConfigContext();
        
        System.out.println("PasswordSyncListener - processing password synchronization requests  from JMS topic '" + this.topicName + "'");
        System.out.println("Note - completed request will be notified under the JMS topic '" + this.notifyTopicName + "'");
        init();
        snoop();
        
    }
    
    /**
     * Constructor - create new instance of Password Synchronizer listener
     */
    public PasswordSyncListener(String newTopicName, String newNotifyTopicName) {
        serviceConfig = new ServiceConfig();
        serviceConfigHashMap = serviceConfig.getAllConfigContext();
        
        this.topicName = newTopicName;
        this.notifyTopicName = newNotifyTopicName;
        System.out.println("PasswordSyncListener - processing password synchronization requests from JMS topic '" + this.topicName + "'");
        System.out.println("Note - completed request will be notified under the JMS topic '" + this.notifyTopicName + "'");
        init();
        snoop();
    }
    
 /*
  * Initializes the JMS settings
  * @exception ex  Exception
  */
    public void init() {
        
        // for future enhancement, use serviceLocator pattern here
        try {
            this.messageFactory = MessageFactory.newInstance();
            this.topicConnectionFactory = new com.sun.messaging.TopicConnectionFactory();
            this.topicConnection = this.topicConnectionFactory.createTopicConnection();
            this.topicSession = this.topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
            this.topic = this.topicSession.createTopic(this.topicName);
        } // try
        catch (Exception ex) {
            ex.printStackTrace();
            System.out.println("Cannot create topics or topic names");
            System.out.println("Connection problem: " + ex.toString());
            if (topicConnection != null) {
                try {
                    topicConnection.close();
                } catch (JMSException moreEx) {
                    moreEx.printStackTrace();
                }
            } // if topicConnection
            System.exit(1);
        } // catch
    } // init()
    
 /*
  * Displays SOAP header
  * @param header SOAP header
  * @exception ex Exception
  */
    private void dumpHeaderContents(SOAPHeader header) {
        
        try {
            Iterator allHeaders = header.examineAllHeaderElements();
            
            while (allHeaders.hasNext()) {
                SOAPHeaderElement headerElement =
                        (SOAPHeaderElement) allHeaders.next();
                Name headerName = headerElement.getElementName();
                System.out.print("<" +
                        headerName.getQualifiedName() + ">");
                System.out.print("actor='" + headerElement.getActor() + "' ");
                System.out.print("mustUnderstand='" +
                        headerElement.getMustUnderstand() + "' ");
                System.out.println("</" + headerName.getQualifiedName() + ">");
            } // while addHeaders.hasNext
        } catch (Exception ex) {
            ex.printStackTrace();
        } // catch
    } // dumpHeaderContents
    
 /*
  * Retrieves SOAP message contents, and displays
  *   in indented XML format
  *
  * @param iterator   Iterator for the SOAP message node
  * @param indent     indent space for displaying XML messages on screen
  */
    private void getContents(Iterator iterator, String indent) {
        
        while (iterator.hasNext()) {
            Node node = (Node) iterator.next();
            SOAPElement element = null;
            Text text = null;
            if (node instanceof SOAPElement) {
                element = (SOAPElement)node;
                Name name = element.getElementName();
                System.out.print(indent + "<" + name.getQualifiedName());
                Iterator attrs = element.getAllAttributes();
                while (attrs.hasNext()){
                    Name attrName = (Name)attrs.next();
                    System.out.print(" " + attrName.getQualifiedName() + "='" +
                            element.getAttributeValue(attrName) + "'");
                } // while attrs.hasNext
                System.out.println(">");
                
                Iterator iter2 = element.getChildElements();
                getContents(iter2, indent + " ");
                System.out.println(indent + "</" + name.getQualifiedName() + ">");
            } // if node instanceof
            else {
                text = (Text) node;
                String content = text.getValue();
                System.out.println(indent + " " + content);
            } // else
        } // while
    } // getContents
    
 /*
  * Processes each JMS message when received from the JMS topic
  * @param message   JMS message in SOAP format
  * @exception ex    Exception
  */
    public void onMessage(Message message) {
        
        try {
            this.timeStamp = new Date().toString();
            MessageFactory messageFactory = MessageFactory.newInstance();
            
            // Should invoke other Web services security patterns when available
            //  to unmarshall encrypted SOAP messages, e.g. Message Inspector pattern
            SOAPMessage soapMessage =
                    MessageTransformer.SOAPMessageFromJMSMessage( message, messageFactory );
            System.out.println(timeStamp + "- Message received! Converting the JMS message to SOAP message...");
            
            SOAPFactory soapFactory = SOAPFactory.newInstance();
            SOAPHeader thisSoapHeader = soapMessage.getSOAPHeader();
            dumpHeaderContents(thisSoapHeader);
            
            SOAPBody thisSoapBody = soapMessage.getSOAPBody();
            Iterator soapContent = thisSoapBody.getChildElements();
            
            System.out.println();
            System.out.println(timeStamp + "- Rendering SOAP Message Content");
            getContents(soapContent, "");
            
            System.out.println("Attachment counts: " + soapMessage.countAttachments());
            Iterator iterator = soapMessage.getAttachments();
            while ( iterator.hasNext() ) {
                AttachmentPart soapAttach = (AttachmentPart) iterator.next();
                String contentType = soapAttach.getContentType();
                String contentId = soapAttach.getContentId();
                if ( contentType.indexOf("text") >=0 ) {
                } // if contentType
            } // while
            
            // take action to notify Password Synchronization Manager about OK status
            TakeAction action = new TakeAction();
            action.init(notifyTopicName);
            String tempTopicName = findApplicationId(topicName);
              
            if (tempTopicName != null) {
                action.publishPwdSyncResult(tempTopicName, "SOAP");
            } else {
                System.out.println("ERROR - Mismatch between applicationId and topicName. Please check pstdMapping.xml");
            }
            
            
        } // try
        catch (Exception ex) {
            try {
                TextMessage textMessage = (TextMessage) message;
                String text = textMessage.getText();
                System.out.println(timeStamp + "- Password sync request in delimited text: " + text);
                // take action to notify Password Synchronization Manager about OK status
                TakeAction action = new TakeAction();
                action.init(notifyTopicName);
                String tempTopicName = findApplicationId(topicName);
                  
                if (tempTopicName != null) {
                    action.publishPwdSyncResult(tempTopicName, "JMS");
                } else {
                    System.out.println("ERROR - Mismatch between applicationId and topicName. Please check pstdMapping.xml");
                }
                
                
            } catch (Exception anotherEx) {
                anotherEx.printStackTrace();
            }
        } // catch
    }
    
 /*
  * Starts listening to the JMS topic
  * @exception ex  JMSException
  */
    private void snoop() {
        
        char           answer = '\0';
        final boolean  NOLOCAL = true;
        
        try {
            topicSubscriber = topicSession.createSubscriber(topic, null, NOLOCAL);
            topicSession.createSubscriber(topic);
            
            topicSubscriber.setMessageListener(this);
            topicConnection.start();
            
            System.out.println("Command Option : Q=quit, then <return>");
            System.out.println();
            inputStreamReader = new InputStreamReader(System.in);
            while (!((answer == 'q') || (answer == 'Q'))) {
                try {
                    answer = (char) inputStreamReader.read();
                } catch (IOException e) {
                    System.out.println("I/O exception: "
                            + e.toString());
                } // catch
            } // while !answer
            
        } // try
        catch (JMSException ex) {
            System.out.println("Cannot subscribe message");
            System.out.println("Exception occurred: " + ex.toString());
            System.exit(1);
        } finally {
            if (topicConnection != null) {
                try {
                    topicConnection.close();
                } catch (JMSException ex) {
                    System.out.println("Cannot close topicConnection");
                    System.out.println("Connection problem: " + ex.toString());
                    System.exit(1);
                }
            }  // if topicConnection
        }// finallly
    }
    
    /**
     * Helper class to look up the applicationId when given a topicName
     *
     * @param String targetTopicName
     * @return String applicationId
     */
    private String findApplicationId(String targetTopicName) {
        
        for(ServiceConfigContext configContext: this.serviceConfigHashMap.values()) {
            if (configContext.getTopicName().equals(targetTopicName)) {
                return configContext.getApplicationId();
            }
        }
        return null;
    }
    
    
    public static void main(String[] args) {
        
        String newTopicName = new String();
        String newNotifyTopicName = new String();
        
        
        // Command syntax helper
        if (args.length != 2) {
            // take default topic name and notify topic name if no param is given at runtime
            new PasswordSyncListener();
        } else {
            newTopicName = args[0];
            newNotifyTopicName = args[1];
            new PasswordSyncListener(newTopicName, newNotifyTopicName);
        }
        
    } // public main
}
