/*
 * Program: PasswordSyncRequest.java
 * Purpose: Helper class to create a password synch request
 * @author: Ray Lai (ray.lai@sun.com)
 * Updated: May 27, 2005, 4:54 PM
 * Copyright:  (c) 2006 by Ray Lai
 * Remarks:    Full program and documentation will be available under developer.java.net
 *             For simplicity, logger class is removed.
 * ============================================================================================
 * Summary:
 *
 * PasswordSyncRequest takes in a user profile object ProvisioningUserProfile, and creates
 * a SPML addRequest to synchronize user account password. it sends to a default topic
 * PROD_FINANCIAL_FRONTOFFICE, if not specified.
 *
 * ============================================================================================
 */

package com.csp.provisioning;

import java.net.URL;
import java.util.Hashtable;
import javax.activation.DataHandler;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import com.sun.messaging.xml.MessageTransformer;
import com.sun.messaging.TopicConnectionFactory;
import com.csp.provisioning.ProvisioningUserProfile;


public class PasswordSyncRequest {
    
    protected TopicConnectionFactory  topicConnectionFactory = null;
    protected TopicConnection         topicConnection = null;
    protected TopicSession            topicSession = null;
    protected Topic                   topic = null;
    protected TopicPublisher          topicPublisher =  null;
    protected Message                 msg = null;
    protected TextMessage             textMsg = null;
    //protected Hashtable               pwdUserProfile = new Hashtable();
    
    protected ProvisioningUserProfile userProfile = null;
    protected String protocolBinding;
    protected String topicName;
    protected String fullName;
    protected String firstName;
    protected String lastName;
    protected String emailAddress;
    protected String userId;
    protected String password;
    
    /**
     * Constructor - Creates a new instance of PasswordSyncRequest 
     *  Default constructor to call. This default will use a default user profile Mary Jo Parker for demo.
     */
    public PasswordSyncRequest() {
        // default values if not specified
        this.protocolBinding = "SOAP";        
        this.topicName = "PROD_FINANCIAL_FRONTOFFICE";
        this.fullName = "Mary Jo Parker";
        this.firstName = "Mary Jo";
        this.lastName = "Parker";
        this.userId = "mjparker";
        this.emailAddress = "mjparker@namredips.com";
        this.topicName = "prod_application1";
        this.password = "secret";
 
        init(topicName);
        
        try {
            createSPMLRequest();
            start();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    
    /**
     * Constructor - Creates a new instance of PasswordSyncRequest 
     */
    public PasswordSyncRequest(ProvisioningUserProfile userProfile, String topicName, String protocolBinding) {
        
        this.protocolBinding = protocolBinding;
        this.topicName = topicName;
        
        this.fullName = userProfile.getFullName();
        this.firstName = userProfile.getFirstName();
        this.lastName = userProfile.getLastName();
        this.userId = userProfile.getUserId();
        this.emailAddress = userProfile.getEmailAddress();
        this.password = userProfile.getToken();
        
        init(this.topicName);
        
        try {
            createSPMLRequest();
            start();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    /**
     * Initializes JMS settings
     *
     * @param String topicName JMS topic name
     * @exception JMSException ex    JMSException
     */
    private void init(String topicName) {
        
        try {
            // Create JMS topic and settings
            //  Can be replaced by ServiceLocator pattern when available
            topicConnectionFactory = new TopicConnectionFactory();
            topicConnection = topicConnectionFactory.createTopicConnection();
            topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
            // topic = null;
            topic = topicSession.createTopic(topicName);
        } catch (JMSException je) {
            je.printStackTrace();
            System.out.println("Cannot create topics or topic names");
            System.out.println("Connection problem: " + je.toString());
            if (topicConnection != null) {
                try {
                    topicConnection.close();
                } // try
                catch (JMSException moreEx) {
                    moreEx.printStackTrace();
                }
            } // catch
            System.exit(1);
        } // catch
    } // init()
    
    /**
     * Create SPML add request message in SOAP, and bind to JMS
     *
     *  This example uses SPML 1.0 syntax for illustration
     *
     * @param TopicSession session TopicSession JMS topic session
     * @param Message message Message JMS message
     * @param Hashtable pwdUserProfile Hashtable user password info for the SPML message
     * @exception JMSException ex Exception JAXM/SAAJ exception
     */
    private void createSPMLRequest() throws Exception {
        
        try {
            // Create a SOAP envelope
            MessageFactory mf = MessageFactory.newInstance();
            SOAPMessage soapMessage = mf.createMessage();
            SOAPPart soapPart = soapMessage.getSOAPPart();
            SOAPEnvelope soapEnvelope = soapPart.getEnvelope();
            SOAPHeader soapHeader = soapMessage.getSOAPHeader();
            SOAPBody soapBody = soapEnvelope.getBody();
            
            // create addRequest SPML message
            Name name = soapEnvelope.createName("addRequest", "spml", "http://www.coresecuritypattern.com");
            SOAPElement element = soapBody.addChildElement(name);
            SOAPBodyElement addRequest = soapBody.addBodyElement(name);
            Name childName = soapEnvelope.createName("xmlns");
            addRequest.addAttribute(childName, "urn:oasis:names:tc:SPML:1:0");
            
            childName = soapEnvelope.createName("spml");
            addRequest.addAttribute(childName, "urn:oasis:names:tc:DSML:2:0:core");
            
            // create identifier
            childName = soapEnvelope.createName("identifier");
            SOAPElement spmlIdentifier = addRequest.addChildElement(childName);
            childName = soapEnvelope.createName("type");
            spmlIdentifier.addAttribute(childName, "urn:oasis:names:tc:SPML:1:0#GUID");
            
            // create user account id
            childName = soapEnvelope.createName("id");
            SOAPElement spmlID = spmlIdentifier.addChildElement(childName);
            spmlIdentifier.addTextNode(this.userId);
            
            // create user account password
            childName = soapEnvelope.createName("attributes");
            SOAPElement attributes = addRequest.addChildElement(childName);
            childName = soapEnvelope.createName("attr", "dsml", "http://www.sun.com/imq");
            
            childName = soapEnvelope.createName("name");
            SOAPElement attr1 = attributes.addChildElement(childName);
            attributes.addAttribute(childName, "objectclass");
            
            childName = soapEnvelope.createName("value");
            SOAPElement attrObjclass = attr1.addChildElement(childName);
            attrObjclass.addTextNode("user");
            
            childName = soapEnvelope.createName("name");
            SOAPElement attr2 = attributes.addChildElement(childName);
            attributes.addAttribute(childName, "fullname");
            
            childName = soapEnvelope.createName("value");
            SOAPElement attrFullname = attr2.addChildElement(childName);
            attrFullname.addTextNode(this.fullName);
            
            childName = soapEnvelope.createName("name");
            SOAPElement attr3 = attributes.addChildElement(childName);
            attributes.addAttribute(childName, "email");
            
            childName = soapEnvelope.createName("value");
            SOAPElement attrEmail = attr3.addChildElement(childName);
            attrEmail.addTextNode(this.emailAddress);
            
            childName = soapEnvelope.createName("name");
            SOAPElement attr4 = attributes.addChildElement(childName);
            attributes.addAttribute(childName, "password");
            
            childName = soapEnvelope.createName("value");
            SOAPElement attrPassword = attr4.addChildElement(childName);
            attrPassword.addTextNode(this.password);
            
            childName = soapEnvelope.createName("name");
            SOAPElement attr5 = attributes.addChildElement(childName);
            attributes.addAttribute(childName, "lastname");
            
            childName = soapEnvelope.createName("value");
            SOAPElement attrLastname = attr5.addChildElement(childName);
            attrLastname.addTextNode(this.lastName);
            
            childName = soapEnvelope.createName("name");
            SOAPElement attr6 = attributes.addChildElement(childName);
            attributes.addAttribute(childName, "firstname");
            
            childName = soapEnvelope.createName("value");
            SOAPElement attrFirstname = attr6.addChildElement(childName);
            attrFirstname.addTextNode(this.firstName);
            
            // Attach a local file
            URL url = new URL("http://localhost:8080");
            
            DataHandler dHandler = new DataHandler(url);
            AttachmentPart soapAttach = soapMessage.createAttachmentPart(dHandler);
            
            soapAttach.setContentType("text/html");
            soapAttach.setContentId("cid-001");
            
            //soapMessage.addAttachmentPart(soapAttach);
            soapMessage.saveChanges();
            
            // Convert SOAP message to JMS
            // System.out.println ("Convert the SPML/SOAP message to JMS message...");
            this.msg = MessageTransformer.SOAPMessageIntoJMSMessage(soapMessage, this.topicSession);
        }  // try
        catch (Exception ex) {
            ex.printStackTrace();
            System.out.println("Exception occurred: " + ex.toString());
        } // catch
    }
    
    /**
     * Create a string in plain text to encapsulate password sync request
     *
     * @return String a string that concatenates userId, fullName, emailAddress, password, lastName, firstName
     */
    private String createPwdRequest() {
        
        String pwdRequest = null;
        
        pwdRequest = this.userId + ":" + this.fullName + ":" + this.emailAddress + ":" + this.password + ":" + this.lastName + ":" + this.firstName;
        return pwdRequest;
    }
    
    /**
     * Start processing SPML message, given the JMS topic name,
     *    and the SPML user password info
     * It is intended that the start() will start and close JMS connection.
     * For better efficiency, create a batch loop to process multiple requests.
     *
     * @param String topicName String JMS topic name
     * @param Message message   Message JMS message content
     * @param Hashtable pwdUserProfile Hashtable user password info for the SPML message
     * @param String protocol
     */
    private void start() {
        
        String pwdRequestText = null;
        
        try {
            topicPublisher = this.topicSession.createPublisher(topic);
            
            if (this.protocolBinding.equals("SOAP")) {
                try {
                //  this.msg = topicSession.createTextMessage();
                    createSPMLRequest();
                    this.topicPublisher.publish(this.msg);
                } catch (Exception ex) {
                    System.out.println("Cannot create SOAP message");
                    System.out.println("Message creation error: " + ex.toString());
                    System.exit(1);
                } // catch
            } else if (this.protocolBinding.equals("JMS")) {
                this.textMsg = this.topicSession.createTextMessage();
                pwdRequestText = createPwdRequest();
                this.textMsg.setText(pwdRequestText);
                this.topicPublisher.publish(textMsg);
            } else {
                System.out.println("Request protocol " + this.protocolBinding + " is not supported");
                System.exit(1);
            } // if protocol
            
        } // try
        catch (JMSException je) {
            je.printStackTrace();
            System.out.println("Cannot publish SOAP message");
            System.out.println("Exception occurred: " + je.toString());
        } finally {
            if (topicConnection != null) {
                try {
                    topicConnection.close();
                } // try topicConnection
                catch (JMSException jex) {
                    jex.printStackTrace();
                    System.out.println("Cannot close topicConnection");
                    System.out.println("Connection problem: " + jex.toString());
                } // catch
            } // if topicConnection
        } // finally
    } // try
    
    /**
     * set protocol binding for the service request
     *
     * @param String protocolBinding
     */
    public void setProtocolBinding(String protocolBinding) {
        this.protocolBinding = protocolBinding;
    }
    /**
    public static void main(String args[]) {
        new PwdSyncRequest();
    }
     ***/
}
