/*
 * Program: ServiceConfig.java
 * Purpose: This service config component factory loads config file,
 *          and creates a hash map for each application service with a service config context
 * @author: Ray Lai (ray.lai@sun.com)
 * Updated: May 27, 2005, 3:39 PM
 * Copyright:  (c) 2006 by Ray Lai
 * Remarks:    Full program and documentation will be available under developer.java.net
 *             For simplicity, logger class is removed.
 * ============================================================================================
 */

package com.csp.provisioning;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;


public class ServiceConfig {
    
    protected LinkedHashMap<String,ServiceConfigContext> serviceConfigHashMap = new LinkedHashMap();
    protected Document doc = null;
    private static final String configFile = "config/pstidMapping.xml";
    protected String requesterId = "passwordSynManagerUser";  // default service requester id for audit log
    protected Log log;
    
    /** Creates a new instance of ServiceConfig */
    public ServiceConfig() {
        log = LogFactory.getLog(ServiceConfig.class.getPackage().getName());
        try {
            if (configFile == null) {
                log.fatal("Invalid Password Synchronizer configuration file name");
            } else {
                SAXBuilder builder = new SAXBuilder(false);
                doc = builder.build(new File(this.getConfigFile()));
                
                initConfig(doc);  // ensure we can get components config even components not initialized
            }
        } catch (IOException ie) {
            log.fatal("ServiceConfig constructor - cannot find file/file not readable");
            ie.printStackTrace();
        } catch (JDOMException je) {
            log.fatal("cannot parse Password Synchronizer config file");
            je.printStackTrace();
        }
    }
    
    /**
     * Get  config file from JVM options
     * if  the file does not exist, use the default one under config/config.xml
     *
     * @return String config file
     */
    private String getConfigFile() {
        String localConfigFile = new String();
        
        localConfigFile = System.getProperty("config.file");
        if (localConfigFile == null) {
            localConfigFile = this.configFile;
            return localConfigFile;
        } else {
            return localConfigFile;
        }
    }
    
    /**
     *  Initialize configuration by loading the pstdMapping.xml into the LinkedHashMap
     *  This will include:
     *  1. Load configFile
     *  2. Extract global config
     *  3. Extract private config for each component into pstdMapping.xml
     *  4. Store private config info in LinkedHashMap
     *
     *  @param String configFile Ulysses config file
     */
    private void initConfig(Document doc) {
        
        setComponentsConfig(doc);
    }
    
    /**
     * Create pstdMapping.xml from LinkedHashMap
     *
     * Assumption - must load config file and create LinkedHashMap first
     *
     * @param Document doc pstdMapping.xml contents
     */
    private synchronized void setComponentsConfig(Document doc) {
        
        String parentElement = "service";
        String applicationIdElement = "applicationId";
        String applicationClassNameElement = "applicationClassName";
        String applicationURIElement = "applicationURI";
        String protocolBindingElement = "protocolBinding";
        String topicNameElement = "topicName";
        int state = com.csp.provisioning.ServiceConfigContext.UNKNOWN_STATE;
        
        String applicationId = new String();
        String applicationClassName = new String();
        String applicationURI = new String();
        String protocolBinding = new String();
        String topicName = new String();
        //String requesterId = new String();
        String requesterId = this.requesterId;
        ServiceConfigContext context = null;
        
        
        Element root = doc.getRootElement();
        List components = root.getChildren(parentElement);
        Iterator i = components.iterator();
        while (i.hasNext()) {
            Element component = (Element)i.next();
            applicationId = component.getChild(applicationIdElement).getText();
            applicationClassName = component.getChild(applicationClassNameElement).getText();
            applicationURI = component.getChild(applicationURIElement).getText();
            protocolBinding = component.getChild(protocolBindingElement).getText();
            topicName = component.getChild(topicNameElement).getText();
            
            //System.out.println("topic name = " + topicName);
            
            context = new ServiceConfigContext(applicationId, applicationClassName, applicationURI, protocolBinding, state, requesterId, topicName);
            
            this.serviceConfigHashMap.put(applicationId, context);
        }
    }
    
    /**
     * Retrieve private config in a list
     *
     * @param String componentName
     * @return List a list containing the private config of a Ulysses component
     */
    public ServiceConfigContext getContext(String applicationId) {
        ServiceConfigContext context;
        
        if (this.serviceConfigHashMap == null) {
            try {
                if (configFile == null) {
                    log.fatal("Invalid  configuration file name");
                } else {
                    SAXBuilder builder = new SAXBuilder(false);
                    Document doc = builder.build(new File(configFile));
                    initConfig(doc);  // ensure we can get components config even components not initialized
                    //dumpComponentMap();
                    context = this.serviceConfigHashMap.get(applicationId);
                    return context;
                }
            } catch (IOException ie) {
                log.fatal("ServiceConfig constructor - cannot find file/file not readable");
                ie.printStackTrace();
            } catch (JDOMException je) {
                log.fatal("cannot parse  config file");
                je.printStackTrace();
            }
            return null;
        } else {
            context = this.serviceConfigHashMap.get(applicationId);
            return context;
        }
    }
    
    /**
     * Fetch service config context of all components
     *
     * @return LinkedHashMap serviceConfigHashMap
     *
     **/
    public LinkedHashMap getAllConfigContext() {
        return this.serviceConfigHashMap;
    }
}
